@@ -2,7 +2,7 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002, 2003, 2004 The ProFTPD Project team
+ * Copyright (c) 2001-2022 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
  *
  * As a special exemption, Public Flood Software/MacGyver aka Habeeb J. Dihu
  * and other respective copyright holders give permission to link this program
@@ -24,10 +24,7 @@
  * the source code for OpenSSL in the source distribution.
  */
 
-/* FTP commands and responses (may not all be implemented)
- *
- * $Id: ftp.h,v 1.11 2004/10/30 20:44:41 castaglia Exp $
- */
+/* FTP commands and responses (may not all be implemented) */
 
 #ifndef PR_FTP_H
 #define PR_FTP_H
@@ -64,6 +61,8 @@
 #define C_RMD	"RMD"		/* Remove a directory */
 #define C_XRMD	"XRMD"		/* Remove a directory */
 #define C_MKD	"MKD"		/* Create a directory */
+#define C_MLSD	"MLSD"		/* List a directory (RFC3659) */
+#define C_MLST	"MLST"		/* List a path (RFC3659) */
 #define C_XMKD	"XMKD"		/* Create a directory */
 #define C_PWD	"PWD"		/* Return current working directory */
 #define C_XPWD	"XPWD"		/* Return current working directory */
@@ -78,6 +77,10 @@
 #define C_FEAT	"FEAT"		/* Request list of server-supported features */
 #define C_OPTS	"OPTS"		/* Specify options for FTP commands */
 #define C_LANG	"LANG"		/* Request a specific language */
+#define C_HOST	"HOST"		/* Request a named server */
+#define C_CLNT	"CLNT"		/* Client-offered identification */
+#define C_RANG	"RANG"		/* Range of bytes to transfer */
+#define C_CSID	"CSID"		/* Client/Server Identification */
 
 /* RFC2228 FTP Security commands */
 #define C_ADAT  "ADAT"		/* Authentication/security data */
@@ -89,6 +92,10 @@
 #define C_PBSZ  "PBSZ"		/* Protection buffer size */
 #define C_PROT  "PROT"		/* Data channel protection level */
 
+/* Proposed commands */
+#define C_MFF	"MFF"		/* Modify File Fact (RFC3659) */
+#define C_MFMT	"MFMT"		/* Modify File Modify-Type (RFC3659) */
+
 #define C_ANY	"*"		/* Special "wildcard" matching command */
 
 /* Command groupings */
@@ -105,7 +112,7 @@
 #define R_125	"125"		/* Data connection already open; starting */
 #define R_150	"150"		/* File status ok; opening data conn */
 #define R_200	"200"		/* 'Generic' command ok */
-#define R_202	"202"		/* Command not implemented, superflous at this site */
+#define R_202	"202"		/* Command not implemented, superfluous at this site */
 #define R_211	"211"		/* System status or system help reply */
 #define R_212	"212"		/* Directory status */
 #define R_213	"213"		/* File status */
