@@ -1,7 +1,6 @@
 /*
  * ProFTPD: mod_lang -- a module for handling the LANG command [RFC2640]
- *
- * Copyright (c) 2006-2011 The ProFTPD Project
+ * Copyright (c) 2006-2017 The ProFTPD Project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,13 +20,11 @@
  * give permission to link this program with OpenSSL, and distribute the
  * resulting executable, without including the source code for OpenSSL in the
  * source distribution.
- *
- * $Id: mod_lang.c,v 1.36 2011/05/23 21:11:56 castaglia Exp $
  */
 
 #include "conf.h"
 
-#define MOD_LANG_VERSION		"mod_lang/0.9"
+#define MOD_LANG_VERSION		"mod_lang/1.1"
 
 #if PROFTPD_VERSION_NUMBER < 0x0001030101
 # error "ProFTPD 1.3.1rc1 or later required"
@@ -53,6 +50,13 @@ static array_header *lang_list = NULL;
 static pr_table_t *lang_aliases = NULL;
 static const char *lang_path = PR_LOCALE_DIR;
 
+static unsigned long lang_opts = 0UL;
+#define LANG_OPT_PREFER_SERVER_ENCODING		0x0001
+#define LANG_OPT_REQUIRE_VALID_ENCODING		0x0002
+
+static int lang_use_encoding = -1;
+static const char *lang_local_charset = NULL, *lang_client_charset = NULL;
+
 /* Support routines
  */
 
@@ -64,7 +68,7 @@ static void lang_feat_add(pool *p) {
     register unsigned int i;
     char **langs;
     size_t feat_strlen = 0;
- 
+
     langs = lang_list->elts;
     for (i = 0; i < lang_list->nelts; i++) {
       char *lang_dup, *tmp;
@@ -89,7 +93,7 @@ static void lang_feat_add(pool *p) {
 
       feat_str = pstrcat(p, feat_str, ";", NULL);
     }
- 
+
     feat_strlen = strlen(feat_str);
 
     /* Trim the trailing semicolon. */
@@ -131,7 +135,7 @@ static const char *lang_bind_domain(void) {
 #ifdef HAVE_LIBINTL_H
   pr_log_debug(DEBUG9, MOD_LANG_VERSION
     ": binding to text domain 'proftpd' using locale path '%s'", lang_path);
-  res = bindtextdomain("proftpd", lang_path); 
+  res = bindtextdomain("proftpd", lang_path);
   if (res == NULL) {
     pr_log_pri(PR_LOG_NOTICE, MOD_LANG_VERSION
       ": unable to bind to text domain 'proftpd' using locale path '%s': %s",
@@ -156,14 +160,14 @@ static int lang_set_lang(pool *p, const char *lang) {
   char *curr_lang;
 
   if (lang_aliases != NULL) {
-    void *v;
+    const void *v;
 
     /* Check to see if the given lang has an alias that has been determined
      * to be acceptable.
      */
 
     v = pr_table_get(lang_aliases, lang, NULL);
-    if (v) {
+    if (v != NULL) {
       pr_log_debug(DEBUG9, MOD_LANG_VERSION ": '%s' is an alias for '%s'",
         lang, (const char *) v);
       lang = v;
@@ -178,12 +182,12 @@ static int lang_set_lang(pool *p, const char *lang) {
 
   if (setlocale(LC_MESSAGES, lang) == NULL) {
     if (errno == ENOENT) {
-      pr_log_pri(PR_LOG_NOTICE, "unknown/unsupported language '%s', ignoring",
-        lang);
+      pr_log_pri(PR_LOG_NOTICE, MOD_LANG_VERSION
+        ": unknown/unsupported language '%s', ignoring", lang);
 
     } else {
-      pr_log_pri(PR_LOG_NOTICE, "unable to set LC_MESSAGES to '%s': %s",
-        lang, strerror(errno));
+      pr_log_pri(PR_LOG_NOTICE, MOD_LANG_VERSION
+        ": unable to set LC_MESSAGES to '%s': %s", lang, strerror(errno));
       return -1;
     }
 
@@ -195,20 +199,20 @@ static int lang_set_lang(pool *p, const char *lang) {
 
     /* Set LC_COLLATE for strcoll(3), for sorted directory listings. */
     if (setlocale(LC_COLLATE, curr_lang) == NULL) {
-      pr_log_pri(PR_LOG_NOTICE, "unable to set LC_COLLATE to '%s': %s",
-        curr_lang, strerror(errno));
+      pr_log_pri(PR_LOG_NOTICE, MOD_LANG_VERSION
+        ": unable to set LC_COLLATE to '%s': %s", curr_lang, strerror(errno));
     }
 
     /* Set LC_CTYPE for conversion, case-sensitive comparisons, and regexes. */
     if (setlocale(LC_CTYPE, curr_lang) == NULL) {
-      pr_log_pri(PR_LOG_NOTICE, "unable to set LC_CTYPE to '%s': %s",
-        curr_lang, strerror(errno));
+      pr_log_pri(PR_LOG_NOTICE, MOD_LANG_VERSION
+        ": unable to set LC_CTYPE to '%s': %s", curr_lang, strerror(errno));
     }
 
     /* Set LC_MONETARY, for handling e.g the Euro symbol. */
     if (setlocale(LC_MONETARY, curr_lang) == NULL) {
-      pr_log_pri(PR_LOG_NOTICE, "unable to set LC_MONETARY to '%s': %s",
-        curr_lang, strerror(errno));
+      pr_log_pri(PR_LOG_NOTICE, MOD_LANG_VERSION
+        ": unable to set LC_MONETARY to '%s': %s", curr_lang, strerror(errno));
     }
   }
 
@@ -237,20 +241,20 @@ static int lang_supported(pool *p, const char *lang) {
   char *lang_dup, **langs;
   int ok = FALSE;
 
-  if (!lang_list) {
+  if (lang_list == NULL) {
     errno = EPERM;
     return -1;
   }
 
   if (lang_aliases != NULL) {
-    void *v;
-    
+    const void *v;
+
     /* Check to see if the given lang has an alias that has been determined
      * to be acceptable.
      */
 
     v = pr_table_get(lang_aliases, lang, NULL);
-    if (v) {
+    if (v != NULL) {
       pr_log_debug(DEBUG9, MOD_LANG_VERSION ": using '%s' as alias for '%s'",
         (const char *) v, lang);
       lang = v;
@@ -285,6 +289,72 @@ static int lang_supported(pool *p, const char *lang) {
   return 0;
 }
 
+/* Lookup/handle any UseEncoding configuration. */
+static int process_encoding_config(int *utf8_client_encoding) {
+  config_rec *c;
+  int strict_encoding;
+
+  c = find_config(main_server->conf, CONF_PARAM, "UseEncoding", FALSE);
+  if (c == NULL) {
+    errno = ENOENT;
+    return -1;
+  }
+
+  if (c->argc == 1) {
+    lang_use_encoding = *((int *) c->argv[0]);
+    if (lang_use_encoding == TRUE) {
+      pr_fs_use_encoding(TRUE);
+
+    } else {
+      pr_encode_disable_encoding();
+      pr_fs_use_encoding(FALSE);
+    }
+
+    return 0;
+  }
+
+  lang_local_charset = c->argv[0];
+  lang_client_charset = c->argv[1];
+  strict_encoding = *((int *) c->argv[2]);
+
+  if (strict_encoding == TRUE) {
+    /* Fold the UseEncoding "strict" keyword functionality into LangOptions;
+     * we want to move people that way anyway.
+     */
+    lang_opts |= LANG_OPT_PREFER_SERVER_ENCODING;
+  }
+
+  if (strcasecmp(lang_client_charset, "UTF8") == 0 ||
+      strcasecmp(lang_client_charset, "UTF-8") == 0) {
+    if (utf8_client_encoding) {
+      *utf8_client_encoding = TRUE;
+    }
+  }
+
+  if (pr_encode_set_charset_encoding(lang_local_charset,
+      lang_client_charset) < 0) {
+    pr_log_pri(PR_LOG_NOTICE, MOD_LANG_VERSION
+      ": error setting local charset '%s', client charset '%s': %s",
+      lang_local_charset, lang_client_charset, strerror(errno));
+    pr_fs_use_encoding(FALSE);
+
+  } else {
+    pr_log_debug(DEBUG3, MOD_LANG_VERSION ": using local charset '%s', "
+      "client charset '%s' for path encoding", lang_local_charset,
+      lang_client_charset);
+    pr_fs_use_encoding(TRUE);
+
+    /* Make sure that gettext() uses the specified charset as well. */
+    if (bind_textdomain_codeset("proftpd", lang_client_charset) == NULL) {
+      pr_log_pri(PR_LOG_NOTICE, MOD_LANG_VERSION
+        ": error setting client charset '%s' for localised messages: %s",
+        lang_client_charset, strerror(errno));
+    }
+  }
+
+  return 0;
+}
+
 /* Configuration handlers
  */
 
@@ -299,19 +369,52 @@ MODRET set_langdefault(cmd_rec *cmd) {
 
 /* usage: LangEngine on|off */
 MODRET set_langengine(cmd_rec *cmd) {
-  int bool;
+  int b;
   config_rec *c;
 
   CHECK_ARGS(cmd, 1);
   CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL);
 
-  bool = get_boolean(cmd, 1);
-  if (bool == -1)
+  b = get_boolean(cmd, 1);
+  if (b == -1) {
     CONF_ERROR(cmd, "expected Boolean parameter");
+  }
 
   c = add_config_param(cmd->argv[0], 1, NULL);
   c->argv[0] = pcalloc(c->pool, sizeof(int));
-  *((int *) c->argv[0]) = bool;
+  *((int *) c->argv[0]) = b;
+
+  return PR_HANDLED(cmd);
+}
+
+/* usage: LangOptions opt1 opt2 ... */
+MODRET set_langoptions(cmd_rec *cmd) {
+  config_rec *c = NULL;
+  register unsigned int i = 0;
+  unsigned long opts = 0UL;
+
+  if (cmd->argc-1 == 0)
+    CONF_ERROR(cmd, "wrong number of parameters");
+
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL);
+
+  c = add_config_param(cmd->argv[0], 1, NULL);
+
+  for (i = 1; i < cmd->argc; i++) {
+    if (strcmp(cmd->argv[i], "PreferServerEncoding") == 0) {
+      opts |= LANG_OPT_PREFER_SERVER_ENCODING;
+
+    } else if (strcmp(cmd->argv[i], "RequireValidEncoding") == 0) {
+      opts |= LANG_OPT_REQUIRE_VALID_ENCODING;
+
+    } else {
+      CONF_ERROR(cmd, pstrcat(cmd->tmp_pool, ": unknown LangOption '",
+        cmd->argv[i], "'", NULL));
+    }
+  }
+
+  c->argv[0] = pcalloc(c->pool, sizeof(unsigned long));
+  *((unsigned long *) c->argv[0]) = opts;
 
   return PR_HANDLED(cmd);
 }
@@ -330,26 +433,47 @@ MODRET set_langpath(cmd_rec *cmd) {
   return PR_HANDLED(cmd);
 }
 
-/* usage: UseEncoding on|off|local-charset client-charset */
+/* usage: UseEncoding on|off|local-charset client-charset ["strict"]*/
 MODRET set_useencoding(cmd_rec *cmd) {
   config_rec *c;
 
   CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL);
 
   if (cmd->argc == 2) {
-    int bool = -1;
+    int use_encoding = -1;
 
-    bool = get_boolean(cmd, 1);
-    if (bool == -1) {
+    use_encoding = get_boolean(cmd, 1);
+    if (use_encoding == -1) {
       CONF_ERROR(cmd, "expected Boolean parameter");
     }
 
     c = add_config_param(cmd->argv[0], 1, NULL);
     c->argv[0] = pcalloc(c->pool, sizeof(int));
-    *((int *) c->argv[0]) = bool;
+    *((int *) c->argv[0]) = use_encoding;
 
-  } else if (cmd->argc == 3) {
-    c = add_config_param_str(cmd->argv[0], 2, cmd->argv[1], cmd->argv[2]);
+  } else if (cmd->argc == 3 ||
+             cmd->argc == 4) {
+
+    if (cmd->argc == 4) {
+      if (strcasecmp(cmd->argv[3], "strict") != 0) {
+        CONF_ERROR(cmd, pstrcat(cmd->tmp_pool, "unknown UseEncoding keyword '",
+          cmd->argv[3], "'", NULL));
+      }
+    }
+
+    c = add_config_param(cmd->argv[0], 3, NULL, NULL, NULL);
+    c->argv[0] = pstrdup(c->pool, cmd->argv[1]);
+    c->argv[1] = pstrdup(c->pool, cmd->argv[2]);
+    c->argv[2] = palloc(c->pool, sizeof(int));
+
+    if (cmd->argc == 4) {
+      /* UseEncoding strict keyword in effect. */
+      *((int *) c->argv[2]) = TRUE;
+
+    } else {
+      /* UseEncoding strict keyword NOT in effect. */
+      *((int *) c->argv[2]) = FALSE;
+    }
 
   } else {
     CONF_ERROR(cmd, "wrong number of parameters");
@@ -370,6 +494,9 @@ MODRET lang_lang(cmd_rec *cmd) {
   if (!dir_check(cmd->tmp_pool, cmd, cmd->group, session.cwd, NULL)) {
     pr_log_debug(DEBUG4, MOD_LANG_VERSION ": LANG command denied by <Limit>");
     pr_response_add_err(R_500, _("Unable to handle command"));
+
+    pr_cmd_set_errno(cmd, EPERM);
+    errno = EPERM;
     return PR_ERROR(cmd);
   }
 
@@ -387,11 +514,17 @@ MODRET lang_lang(cmd_rec *cmd) {
     pr_log_debug(DEBUG7, MOD_LANG_VERSION ": assuming language files are "
       "unavailable after login, denying LANG command");
     pr_response_add_err(R_500, _("Unable to handle command"));
+
+    pr_cmd_set_errno(cmd, EPERM);
+    errno = EPERM;
     return PR_ERROR(cmd);
   }
 
   if (cmd->argc > 2) {
-    pr_response_add_err(R_501, _("Invalid number of arguments"));
+    pr_response_add_err(R_501, _("Invalid number of parameters"));
+
+    pr_cmd_set_errno(cmd, EINVAL);
+    errno = EINVAL;
     return PR_ERROR(cmd);
   }
 
@@ -412,18 +545,22 @@ MODRET lang_lang(cmd_rec *cmd) {
 
   if (lang_supported(cmd->tmp_pool, cmd->argv[1]) < 0) {
     pr_log_debug(DEBUG3, MOD_LANG_VERSION ": language '%s' unsupported: %s",
-      cmd->argv[1], strerror(errno));
-    pr_response_add_err(R_504, _("Language %s not supported"), cmd->argv[1]);
+      (char *) cmd->argv[1], strerror(errno));
+    pr_response_add_err(R_504, _("Language %s not supported"),
+      (char *) cmd->argv[1]);
+
+    pr_cmd_set_errno(cmd, EPERM);
+    errno = EPERM;
     return PR_ERROR(cmd);
   }
 
   pr_log_debug(DEBUG7, MOD_LANG_VERSION
-    ": setting to client-requested language '%s'", cmd->argv[1]);
+    ": setting to client-requested language '%s'", (char *) cmd->argv[1]);
 
   if (lang_set_lang(cmd->tmp_pool, cmd->argv[1]) < 0) {
     pr_log_pri(PR_LOG_NOTICE, MOD_LANG_VERSION
-      ": unable to use client-requested language '%s': %s", cmd->argv[1],
-      strerror(errno));
+      ": unable to use client-requested language '%s': %s",
+      (char *) cmd->argv[1], strerror(errno));
     pr_log_pri(PR_LOG_NOTICE, MOD_LANG_VERSION
       ": using LangDefault '%s' instead", lang_default);
 
@@ -450,9 +587,14 @@ MODRET lang_lang(cmd_rec *cmd) {
   return PR_HANDLED(cmd);
 }
 
+MODRET lang_post_pass(cmd_rec *cmd) {
+  (void) process_encoding_config(NULL);
+  return PR_DECLINED(cmd);
+}
+
 MODRET lang_utf8(cmd_rec *cmd) {
   register unsigned int i;
-  int bool;
+  int use_utf8;
   const char *curr_encoding;
   char *method;
 
@@ -462,97 +604,168 @@ MODRET lang_utf8(cmd_rec *cmd) {
    * logging.
    */
   for (i = 0; method[i]; i++) {
-    if (method[i] == '_')
+    if (method[i] == '_') {
       method[i] = ' ';
+    }
   }
 
   if (cmd->argc != 2) {
     pr_response_add_err(R_501, _("'%s' not understood"), method);
+
+    pr_cmd_set_errno(cmd, EINVAL);
+    errno = EINVAL;
     return PR_ERROR(cmd);
   }
 
-  bool = get_boolean(cmd, 1);
-  if (bool < 0) {
+  use_utf8 = get_boolean(cmd, 1);
+  if (use_utf8 < 0) {
     pr_response_add_err(R_501, _("'%s' not understood"), method);
+
+    pr_cmd_set_errno(cmd, EINVAL);
+    errno = EINVAL;
     return PR_ERROR(cmd);
   }
 
   curr_encoding = pr_encode_get_encoding();
+  if (curr_encoding != NULL) {
+    pr_log_debug(DEBUG9, MOD_LANG_VERSION
+      ": Handling OPTS UTF8 %s (current encoding is '%s')",
+      (char *) cmd->argv[1], curr_encoding);
+
+  } else {
+    pr_log_debug(DEBUG9, MOD_LANG_VERSION
+      ": Handling OPTS UTF8 %s (encoding currently disabled)",
+      (char *) cmd->argv[1]);
+  }
 
   if (pr_encode_is_utf8(curr_encoding) == TRUE) {
-    if (bool) {
+    if (use_utf8) {
       /* Client requested that we use UTF8, and we already are.  Nothing
        * more needs to be done.
        */
       pr_response_add(R_200, _("UTF8 set to on"));
 
     } else {
-      config_rec *c;
-
-      /* Client requested that we NOT use UTF8, and we are.  Need to disable
-       * encoding, then, unless the UseEncoding setting dictates that we
-       * must.
+      /* Client requested that we NOT use UTF8 (i.e. "OPTS UTF8 off"), and
+       * we are.  Need to disable encoding, then, unless the
+       * LangOptions/UseEncoding settings dictate that we MUST use UTF8.
        */
 
-      c = find_config(main_server->conf, CONF_PARAM, "UseEncoding", FALSE);
-      if (c) {
-        /* We have explicit UseEncoding instructions; we cannot change
-         * the encoding use as requested by the client.
+      if (lang_use_encoding == TRUE) {
+        /* We have explicit instructions; we cannot change the encoding use as
+         * requested by the client.
          */
         pr_log_debug(DEBUG5, MOD_LANG_VERSION
-          ": unable to accept 'OPTS UTF8 off' due to UseEncoding directive in "
-          "config file");
+          ": unable to accept 'OPTS UTF8 off' due to LangOptions/UseEncoding "
+          "directive in config file");
         pr_response_add_err(R_451, _("Unable to accept %s"), method);
+
+        pr_cmd_set_errno(cmd, EPERM);
+        errno = EPERM;
         return PR_ERROR(cmd);
 
-      } else {
-        pr_log_debug(DEBUG5, MOD_LANG_VERSION
-          ": disabling use of UTF8 encoding as per client's request");
+      } else if (lang_local_charset != NULL &&
+                 lang_client_charset != NULL) {
 
-        /* No explicit UseEncoding instructions; we can turn off encoding. */
-        pr_encode_disable_encoding();
-        pr_fs_use_encoding(FALSE);
+        /* UseEncoding configured with specific charsets, and the client
+         * requested that we turn UTF8 support off.  Easy enough; just
+         * (re)set the encoding to use the configured charsets.
+         */
 
+        if (lang_opts & LANG_OPT_PREFER_SERVER_ENCODING) {
+          /* We have explicit instructions; we cannot change the encoding
+           * use as requested by the client.
+           */
+          pr_log_debug(DEBUG5, MOD_LANG_VERSION
+            ": unable to accept 'OPTS UTF8 off' due to "
+            "LangOptions/UseEncoding directive in config file");
+          pr_response_add_err(R_451, _("Unable to accept %s"), method);
+
+          pr_cmd_set_errno(cmd, EPERM);
+          errno = EPERM;
+          return PR_ERROR(cmd);
+        }
+
+        if (pr_encode_set_charset_encoding(lang_local_charset,
+            lang_client_charset) < 0) {
+
+          pr_log_pri(PR_LOG_NOTICE, MOD_LANG_VERSION
+            ": error setting local charset '%s', client charset '%s': %s",
+            lang_local_charset, lang_client_charset, strerror(errno));
+          pr_fs_use_encoding(FALSE);
+          pr_response_add_err(R_451, _("Unable to accept %s"), method);
+
+          pr_cmd_set_errno(cmd, EPERM);
+          errno = EPERM;
+          return PR_ERROR(cmd);
+        }
+
+        pr_log_debug(DEBUG3, MOD_LANG_VERSION ": using local charset '%s', "
+          "client charset '%s' for path encoding", lang_local_charset,
+          lang_client_charset);
+        pr_fs_use_encoding(TRUE);
         pr_response_add(R_200, _("UTF8 set to off"));
+        return PR_HANDLED(cmd);
       }
+
+      pr_log_debug(DEBUG5, MOD_LANG_VERSION
+        ": disabling use of UTF8 encoding as per client's request");
+
+      /* No explicit UseEncoding instructions; we can turn off encoding. */
+      pr_encode_disable_encoding();
+      pr_fs_use_encoding(FALSE);
+      pr_response_add(R_200, _("UTF8 set to off"));
     }
 
   } else {
-
-    if (bool) {
-      config_rec *c;
-
-      /* Client requested that we use UTF8, and we currently are not.
-       * Enable UTF8 encoding, unless the UseEncoding setting dictates that
-       * we cannot.
+    if (use_utf8) {
+      /* Client requested that we use UTF8 (i.e. "OPTS UTF8 on"), and we
+       * currently are not.  Enable UTF8 encoding, unless the
+       * LangOptions/UseEncoding setting dictates that we cannot.
        */
 
-      c = find_config(main_server->conf, CONF_PARAM, "UseEncoding", FALSE);
-      if (c) {
-        /* We have explicit UseEncoding instructions. */
+      if (lang_use_encoding == FALSE) {
+        /* We have explicit instructions. */
         pr_log_debug(DEBUG5, MOD_LANG_VERSION
-          ": unable to accept 'OPTS UTF8 on' due to UseEncoding directive in "
-          "config file");
+          ": unable to accept 'OPTS UTF8 on' due to LangOptions/UseEncoding "
+          "directive in config file");
         pr_response_add_err(R_451, _("Unable to accept %s"), method);
+
+        pr_cmd_set_errno(cmd, EPERM);
+        errno = EPERM;
         return PR_ERROR(cmd);
 
-      } else {
+      } else if (lang_opts & LANG_OPT_PREFER_SERVER_ENCODING) {
+        /* We have explicit instructions; we cannot change the encoding use
+         * as requested by the client.
+         */
         pr_log_debug(DEBUG5, MOD_LANG_VERSION
-          ": enabling use of UTF8 encoding as per client's request");
+          ": unable to accept 'OPTS UTF8 on' due to LangOptions/UseEncoding "
+          "directive in config file");
+        pr_response_add_err(R_451, _("Unable to accept %s"), method);
 
-        /* No explicit UseEncoding instructions; we can turn on encoding. */
-        if (pr_encode_enable_encoding("UTF-8") < 0) {
-          pr_log_debug(DEBUG3, MOD_LANG_VERSION
-            ": error enabling UTF8 encoding: %s", strerror(errno));
-          pr_response_add_err(R_451, _("Unable to accept %s"), method);
-          return PR_ERROR(cmd);
-
-        } else {
-          pr_fs_use_encoding(TRUE);
-          pr_response_add(R_200, _("UTF8 set to on"));
-        }
+        pr_cmd_set_errno(cmd, EPERM);
+        errno = EPERM;
+        return PR_ERROR(cmd);
       }
 
+      pr_log_debug(DEBUG5, MOD_LANG_VERSION
+        ": enabling use of UTF8 encoding as per client's request");
+
+      /* No explicit UseEncoding instructions; we can turn on encoding. */
+      if (pr_encode_enable_encoding("UTF-8") < 0) {
+        pr_log_debug(DEBUG3, MOD_LANG_VERSION
+          ": error enabling UTF8 encoding: %s", strerror(errno));
+        pr_response_add_err(R_451, _("Unable to accept %s"), method);
+
+        pr_cmd_set_errno(cmd, EPERM);
+        errno = EPERM;
+        return PR_ERROR(cmd);
+      }
+
+      pr_fs_use_encoding(TRUE);
+      pr_response_add(R_200, _("UTF8 set to on"));
+
     } else {
       /* Client requested that we not use UTF8, and we are not.  Nothing more
        * needs to be done.
@@ -572,9 +785,7 @@ static void lang_postparse_ev(const void *event_data, void *user_data) {
   config_rec *c;
   DIR *dirh;
   server_rec *s;
-#ifdef HAVE_LIBINTL_H
   const char *locale_path = NULL;
-#endif
 
   c = find_config(main_server->conf, CONF_PARAM, "LangEngine", FALSE);
   if (c) {
@@ -731,6 +942,8 @@ static void lang_postparse_ev(const void *event_data, void *user_data) {
         pr_log_pri(PR_LOG_NOTICE, MOD_LANG_VERSION
           ": LangDefault '%s', configured for server '%s', is not a supported "
           "language, removing", lang, s->ServerName);
+        pr_log_pri(PR_LOG_NOTICE, MOD_LANG_VERSION
+          ": Perhaps proftpd has not yet been translated into '%s'", lang);
         remove_config(s->conf, "LangDefault", FALSE);
       }
     }
@@ -742,6 +955,7 @@ static void lang_postparse_ev(const void *event_data, void *user_data) {
 
 static void lang_restart_ev(const void *event_data, void *user_data) {
   destroy_pool(lang_pool);
+  lang_curr = LANG_DEFAULT_LANG;
   lang_list = NULL;
   lang_aliases = NULL;
 
@@ -764,16 +978,19 @@ static int lang_init(void) {
 
 static int lang_sess_init(void) {
   config_rec *c;
+  int res, utf8_client_encoding = FALSE;
 
   c = find_config(main_server->conf, CONF_PARAM, "LangEngine", FALSE);
-  if (c)
+  if (c != NULL) {
     lang_engine = *((int *) c->argv[0]);
+  }
 
-  if (!lang_engine)
+  if (lang_engine == FALSE) {
     return 0;
+  }
 
   c = find_config(main_server->conf, CONF_PARAM, "LangDefault", FALSE);
-  if (c) {
+  if (c != NULL) {
     char *lang;
 
     lang = c->argv[0];
@@ -802,71 +1019,52 @@ static int lang_sess_init(void) {
      * could not be found for some reason), we should still have an entry for
      * the current language.
      */
-    if (!lang_list) {
+    if (lang_list == NULL) {
       lang_list = make_array(lang_pool, 1, sizeof(char *));
     }
 
     if (lang_list->nelts == 0) {
       *((char **) push_array(lang_list)) = pstrdup(lang_pool, lang_curr);
-    } 
+    }
   }
 
-  c = find_config(main_server->conf, CONF_PARAM, "UseEncoding", FALSE);
-  if (c) {
-    if (c->argc == 1) {
-      int bool;
+  c = find_config(main_server->conf, CONF_PARAM, "LangOptions", FALSE);
+  while (c != NULL) {
+    unsigned long opts = 0;
 
-      bool = *((int *) c->argv[0]);
-      if (bool) {
-        pr_feat_add("UTF8");
-        pr_fs_use_encoding(TRUE);
+    pr_signals_handle();
 
-      } else {
-        pr_encode_disable_encoding();
-        pr_fs_use_encoding(FALSE);
-      }
+    opts = *((unsigned long *) c->argv[0]);
+    lang_opts |= opts;
 
-    } else {
-      char *local_charset, *client_charset;
+    c = find_config_next(c, c->next, CONF_PARAM, "LangOptions", FALSE);
+  }
 
-      local_charset = c->argv[0];
-      client_charset = c->argv[1];
+  if (lang_opts & LANG_OPT_REQUIRE_VALID_ENCODING) {
+    unsigned long encoding_policy;
 
-      if (pr_encode_set_charset_encoding(local_charset, client_charset) < 0) {
-        pr_log_pri(PR_LOG_NOTICE, MOD_LANG_VERSION
-          ": error setting local charset '%s', client charset '%s': %s",
-          local_charset, client_charset, strerror(errno));
-        pr_fs_use_encoding(FALSE);
+    encoding_policy = pr_encode_get_policy();
+    encoding_policy |= PR_ENCODE_POLICY_FL_REQUIRE_VALID_ENCODING;
 
-      } else {
-        pr_log_debug(DEBUG3, MOD_LANG_VERSION ": using local charset '%s', "
-          "client charset '%s' for path encoding", local_charset,
-          client_charset);
-        pr_fs_use_encoding(TRUE);
+    pr_encode_set_policy(encoding_policy);
+  }
 
-        /* Make sure that gettext() uses the specified charset as well. */
-        if (bind_textdomain_codeset("proftpd", client_charset) == NULL) {
-          pr_log_pri(PR_LOG_NOTICE, MOD_LANG_VERSION
-            ": error setting client charset '%s' for localised messages: %s",
-            client_charset, strerror(errno));
-        }
-
-        /* If the client charset specified happens to be UTF8, we need to
-         * make sure it shows up in FEAT.
-         */
-        if (strcasecmp(client_charset, "UTF8") == 0 ||
-            strcasecmp(client_charset, "UTF-8") == 0) {
-          pr_feat_add("UTF8");
-        }
-      }
-    }
-
-  } else {
+  res = process_encoding_config(&utf8_client_encoding);
+  if (res < 0 &&
+      errno == ENOENT) {
     /* Default is to use UTF8. */
-    pr_feat_add("UTF8");
     pr_fs_use_encoding(TRUE);
   }
 
+  /* If the PreferServerEncoding LangOption is not set, OR if the encoding
+   * configured explicitly requests UTF8 from the client, then we can list
+   * UTF8 in the FEAT response.
+   */
+  if (!(lang_opts & LANG_OPT_PREFER_SERVER_ENCODING) ||
+      utf8_client_encoding == TRUE) {
+    pr_feat_add("UTF8");
+  }
+
   /* Configure a proper FEAT line, for our supported languages and our
    * default language.
    */
@@ -881,6 +1079,7 @@ static int lang_sess_init(void) {
 static conftable lang_conftab[] = {
   { "LangDefault",	set_langdefault,	NULL },
   { "LangEngine",	set_langengine,		NULL },
+  { "LangOptions",	set_langoptions,	NULL },
   { "LangPath",		set_langpath,		NULL },
   { "UseEncoding",	set_useencoding,	NULL },
   { NULL }
@@ -889,6 +1088,7 @@ static conftable lang_conftab[] = {
 static cmdtable lang_cmdtab[] = {
   { CMD,	C_LANG,			G_NONE,	lang_lang,	FALSE,	FALSE },
   { CMD,	C_OPTS "_UTF8",		G_NONE,	lang_utf8,	FALSE,	FALSE },
+  { POST_CMD,	C_PASS,			G_NONE,	lang_post_pass,	FALSE,	FALSE },
   { 0, NULL }
 };
 
