@@ -1,6 +1,6 @@
 /*
  * ProFTPD - FTP server daemon
- * Copyright (c) 2001-2022 The ProFTPD Project team
+ * Copyright (c) 2001-2023 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -137,7 +137,7 @@ conn_t *pr_ipbind_get_listening_conn(server_rec *server,
         use_elt = TRUE;
       }
 
-      if (use_elt) { 
+      if (use_elt == TRUE) {
         lr->claimed = TRUE;
         return lr->conn;
       }
@@ -151,7 +151,7 @@ conn_t *pr_ipbind_get_listening_conn(server_rec *server,
     listening_conn_list = xaset_create(listening_conn_pool, NULL);
   }
 
-  p = make_sub_pool(listening_conn_pool); 
+  p = make_sub_pool(listening_conn_pool);
   pr_pool_tag(p, "Listening conn subpool");
 
   l = pr_inet_create_conn(p, -1, addr, port, FALSE);
@@ -1208,10 +1208,10 @@ unsigned int pr_namebind_count(server_rec *srv) {
     return 0;
   }
 
-  ipbind = pr_ipbind_find(srv->addr, srv->ServerPort, FALSE); 
+  ipbind = pr_ipbind_find(srv->addr, srv->ServerPort, FALSE);
   if (ipbind != NULL &&
       ipbind->ib_namebinds != NULL) {
-    count = ipbind->ib_namebinds->nelts; 
+    count = ipbind->ib_namebinds->nelts;
   }
 
   return count;
@@ -1562,7 +1562,7 @@ static int init_standalone_bindings(void) {
   /* If a port is set to zero, the address/port is not bound to a socket
    * at all.
    */
-  if (main_server->ServerPort) {
+  if (main_server->ServerPort > 0) {
     /* If SocketBindTight is off, then pr_inet_create_conn() will
      * create and bind to a wildcard socket.  However, should it be an
      * IPv4 or an IPv6 wildcard socket?
@@ -1600,6 +1600,13 @@ static int init_standalone_bindings(void) {
     is_default = TRUE;
   }
 
+  if (main_server->ServerPort == 0) {
+    /* If there is no server port, then this vhost cannot be treated as the
+     * DefaultServer.
+     */
+    is_default = FALSE;
+  }
+
   if (main_server->ServerPort > 0 ||
       is_default == TRUE) {
 
@@ -1652,6 +1659,13 @@ static int init_standalone_bindings(void) {
         is_default = TRUE;
       }
 
+      if (serv->ServerPort == 0) {
+        /* If there is no server port, then this vhost cannot be treated as the
+         * DefaultServer.
+         */
+        is_default = FALSE;
+      }
+
       if (serv->ServerPort > 0) {
         if (SocketBindTight == FALSE) {
 #ifdef PR_USE_IPV6
@@ -1694,7 +1708,7 @@ static int init_standalone_bindings(void) {
             __LINE__, serv->ServerAddress, strerror(errno));
         }
 
-      } else if (is_default) {
+      } else if (is_default == TRUE) {
         serv->listen = NULL;
 
         res = pr_ipbind_create(serv, serv->addr, serv->ServerPort);
