@@ -1,9 +1,7 @@
 /*
  * ProFTPD - FTP server daemon
- * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
- *  
+ * Copyright (c) 2001-2023 The ProFTPD Project team
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -16,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.
  *
  * As a special exemption, Public Flood Software/MacGyver aka Habeeb J. Dihu
  * and other respective copyright holders give permission to link this program
@@ -24,14 +22,10 @@
  * the source code for OpenSSL in the source distribution.
  */
 
-/* ProFTPD Controls command-line client
- *
- * $Id: ftpdctl.c,v 1.3 2005/08/01 03:16:09 castaglia Exp $
- */
+/* ProFTPD Controls command-line client */
 
 #include "conf.h"
-
-#include <signal.h>
+#include "privs.h"
 
 #ifdef HAVE_GETOPT_H
 # include <getopt.h>
@@ -41,6 +35,7 @@ static const char *program = "ftpdctl";
 
 /* NOTE: these empty stubs are needed for proper linking.  What a mess. */
 
+unsigned char is_master;
 session_t session;
 server_rec *main_server = NULL;
 
@@ -48,47 +43,161 @@ void *get_param_ptr(xaset_t *set, const char *name, int recurse) {
   return NULL;
 }
 
-char *pr_inet_validate(char *buf) {
-  return NULL;
-}
-
 void pr_alarms_block(void) {
 }
 
 void pr_alarms_unblock(void) {
 }
 
+gid_t pr_auth_name2gid(pool *p, const char *name) {
+  return (gid_t) -1;
+}
+
+uid_t pr_auth_name2uid(pool *p, const char *name) {
+  return (uid_t) -1;
+}
+
+void pr_event_generate(const char *event, const void *event_data) {
+  (void) event;
+  (void) event_data;
+}
+
+int pr_event_listening(const char *event) {
+  return -1;
+}
+
+void pr_fs_fadvise(int fd, off_t off, off_t len, int advice) {
+}
+
+int pr_fs_get_usable_fd(int fd) {
+  return -1;
+}
+
+struct tm *pr_localtime(pool *p, const time_t *t) {
+  struct tm *tm;
+
+#if defined(HAVE_LOCALTIME_R)
+  if (p == NULL) {
+    errno = EINVAL;
+    return NULL;
+  }
+
+  tm = localtime_r(t, pcalloc(p, sizeof(struct tm)));
+#else
+  tm = localtime(t);
+#endif /* HAVE_LOCALTIME_R */
+
+  return tm;
+}
+
+int pr_privs_root(const char *file, int lineno) {
+  return 0;
+}
+
+int pr_privs_relinquish(const char *file, int lineno) {
+  return 0;
+}
+
+long pr_random_next(long min, long max) {
+  return 1;
+}
+
 void pr_signals_block(void) {
 }
 
 void pr_signals_unblock(void) {
 }
 
-char *sstrcat(char *dest, const char *src, size_t n) {
-  register char *d;
-
-  for (d = dest; *d && n > 1; d++, n--) ;
-
-  while(n-- > 1 && *src)
-    *d++ = *src++;
-
-  *d = 0;
-  return dest;
+void pr_signals_handle(void) {
 }
 
-char *sstrncpy(char *dest, const char *src, size_t n) {
-  register char *d = dest;
+int pr_snprintf(char *buf, size_t bufsz, const char *fmt, ...) {
+  va_list msg;
+  int res;
 
-  if (!dest)
-    return NULL;
+  va_start(msg, fmt);
+  res = pr_vsnprintf(buf, bufsz, fmt, msg);
+  va_end(msg);
+
+  return res;
+}
+
+pr_table_t *pr_table_alloc(pool *p, int flags) {
+  errno = EPERM;
+  return NULL;
+}
+
+int pr_table_add(pr_table_t *tab, const char *k, const void *v, size_t sz) {
+  errno = EPERM;
+  return -1;
+}
+
+int pr_table_count(pr_table_t *tab) {
+  errno = EPERM;
+  return -1;
+}
+
+int pr_table_empty(pr_table_t *tab) {
+  errno = EPERM;
+  return -1;
+}
+
+int pr_table_exists(pr_table_t *tab, const char *k) {
+  errno = EPERM;
+  return -1;
+}
+
+int pr_table_free(pr_table_t *tab) {
+  errno = EPERM;
+  return -1;
+}
+
+const void *pr_table_get(pr_table_t *tab, const char *k, size_t *sz) {
+  errno = EPERM;
+  return NULL;
+}
+
+const void *pr_table_remove(pr_table_t *tab, const char *k, size_t *sz) {
+  errno = EPERM;
+  return NULL;
+}
+
+int pr_table_set(pr_table_t *tab, const char *k, const void *v, size_t sz) {
+  errno = EPERM;
+  return -1;
+}
+
+int pr_trace_msg(const char *channel, int level, const char *fmt, ...) {
+  errno = ENOSYS;
+  return -1;
+}
+
+int pr_vsnprintf(char *buf, size_t bufsz, const char *fmt, va_list msg) {
+  return vsnprintf(buf, bufsz, fmt, msg);
+}
+
+int sstrncpy(char *dst, const char *src, size_t n) {
+  register char *d = dst;
+  int res = 0;
+
+  if (dst == NULL) {
+    errno = EINVAL;
+    return -1;
+  }
+
+  if (n == 0) {
+    return 0;
+  }
 
   if (src && *src) {
-    for (; *src && n > 1; n--)
+    for (; *src && n > 1; n--) {
       *d++ = *src++;
+      res++;
+    }
   }
 
   *d = '\0';
-  return dest;
+  return res;
 }
 
 #ifdef PR_USE_CTRLS
@@ -99,7 +208,7 @@ static RETSIGTYPE sig_pipe(int sig) {
 }
 
 static void usage(void) {
-  fprintf(stdout, "usage: %s [options]\n", program);
+  fprintf(stdout, "usage: %s [options] action [...]\n", program);
   fprintf(stdout, "  -h\tdisplays this message\n");
   fprintf(stdout, "  -s\tspecify an alternate local socket\n");
   fprintf(stdout, "  -v\tdisplays more verbose information\n");
@@ -108,23 +217,15 @@ static void usage(void) {
 }
 
 int main(int argc, char *argv[]) {
-  unsigned char verbose = FALSE;
-  char **respargv = NULL;
-  const char *cmdopts = "hs:v";
-
   register int i = 0;
-  char *socket_file = PR_RUN_DIR "/proftpd.sock";
+  unsigned char verbose = FALSE;
+  const char *cmdopts = "hs:v";
+  char *action, **respargv = NULL, *socket_file = PR_RUN_DIR "/proftpd.sock";
   int sockfd = -1, optc = 0, status = 0, respargc = 0;
   unsigned int reqargc = 0;
   pool *ctl_pool = NULL;
   array_header *reqargv = NULL;
 
-  /* Make sure we were called with at least one argument. */
-  if (argc-1 < 1) {
-    fprintf(stdout, "%s: missing required arguments\n", program);
-    exit(1);
-  }
-
   /* Set the POSIXLY_CORRECT environment variable, so that control handlers
    * can themselves have optional flags.
    */
@@ -161,6 +262,12 @@ int main(int argc, char *argv[]) {
     }
   }
 
+  /* Make sure we were called with at least one argument. */
+  if (argv[optind] == NULL) {
+    fprintf(stdout, "%s: missing required action\n", program);
+    exit(1);
+  }
+
   signal(SIGPIPE, sig_pipe);
 
   /* Allocate some memory for proftpd objects. */
@@ -169,9 +276,14 @@ int main(int argc, char *argv[]) {
   reqargv = make_array(ctl_pool, 0, sizeof(char *));
 
   /* Process the command-line args into an array_header. */
+
+  action = argv[optind++];
+
   for (i = optind; i < argc; i++) {
-    if (verbose)
+    if (verbose == TRUE) {
       fprintf(stdout, "%s: adding \"%s\" to reqargv\n", program, argv[i]);
+    }
+
     *((char **) push_array(reqargv)) = pstrdup(ctl_pool, argv[i]);
     reqargc++;
   }
@@ -180,30 +292,40 @@ int main(int argc, char *argv[]) {
   *((char **) push_array(reqargv)) = NULL;
 
   /* Open a connection to the socket maintained by mod_ctrls. */
-  if (verbose)
+  if (verbose == TRUE) {
     fprintf(stdout, "%s: contacting server using '%s'\n", program,
       socket_file);
+  }
 
   sockfd = pr_ctrls_connect(socket_file);
   if (sockfd < 0) {
     fprintf(stderr, "%s: error contacting server using '%s': %s\n", program,
       socket_file, strerror(errno));
+    destroy_pool(ctl_pool);
     exit(1);
   }
 
-  if (verbose)
-    fprintf(stdout, "%s: sending control request\n", program);
+  if (verbose == TRUE) {
+    fprintf(stdout, "%s: sending '%s' control request (%u args)\n", program,
+      action, reqargc);
+  }
 
-  if (pr_ctrls_send_msg(sockfd, 0, reqargc, (char **) reqargv->elts) < 0) {
+  if (pr_ctrls_send_request(ctl_pool, sockfd, action, reqargc,
+      (char **) reqargv->elts) < 0) {
     fprintf(stderr, "%s: error sending request: %s\n", program,
       strerror(errno));
+
+    (void) close(sockfd);
+    destroy_pool(ctl_pool);
+
     exit(1);
   }
 
   /* Read and display the responses. */
 
-  if (verbose)
+  if (verbose == TRUE) {
     fprintf(stdout, "%s: receiving control response\n", program);
+  }
 
   /* Manually set errno to this value, so that if an error (like a segfault)
    * occurs, the error string displayed to the user is more indicative of
@@ -211,24 +333,37 @@ int main(int argc, char *argv[]) {
    */
   errno = EPERM;
 
-  if ((respargc = pr_ctrls_recv_response(ctl_pool, sockfd, &status,
-      &respargv)) < 0) {
+  respargc = pr_ctrls_recv_response(ctl_pool, sockfd, &status, &respargv);
+  if (respargc < 0) {
     fprintf(stdout, "%s: error receiving response: %s\n", program,
       strerror(errno));
+
+    (void) close(sockfd);
+    destroy_pool(ctl_pool);
     exit(1);
   }
 
+  if (verbose == TRUE) {
+    fprintf(stdout, "%s: received response status: %d\n", program,
+      status);
+  }
+
   if (respargv != NULL) {
-    for (i = 0; i < respargc; i++)
+    for (i = 0; i < respargc; i++) {
       fprintf(stdout, "%s: %s\n", program, respargv[i]);
+    }
 
-  } else
+  } else {
     fprintf(stdout, "%s: no response from server\n", program);
+  }
 
+  (void) close(sockfd);
   destroy_pool(ctl_pool);
-  ctl_pool = NULL;
 
-  return 0;
+  /* Controls action status values are zero or negative, thus we need to
+   * convert them to appropriate Unix exit status values.
+   */
+  return (status * -1);
 }
 
 #else
@@ -239,5 +374,4 @@ int main(int argc, char *argv[]) {
   printf("  Please recompile proftpd using --enable-ctrls\n");
   return 1;
 }
-
 #endif /* PR_USE_CTRLS */
