@@ -165,6 +165,7 @@ sub mlsd_ok_raw_active {
 
     AuthUserFile => $setup->{auth_user_file},
     AuthGroupFile => $setup->{auth_group_file},
+    AuthOrder => 'mod_auth_file.c',
 
     IfModules => {
       'mod_delay.c' => {
@@ -313,6 +314,7 @@ sub mlsd_ok_raw_passive {
 
     AuthUserFile => $auth_user_file,
     AuthGroupFile => $auth_group_file,
+    AuthOrder => 'mod_auth_file.c',
 
     IfModules => {
       'mod_delay.c' => {
@@ -463,6 +465,7 @@ sub mlsd_fails_file {
 
     AuthUserFile => $auth_user_file,
     AuthGroupFile => $auth_group_file,
+    AuthOrder => 'mod_auth_file.c',
 
     IfModules => {
       'mod_delay.c' => {
@@ -584,6 +587,7 @@ sub mlsd_ok_cwd_dir {
 
     AuthUserFile => $auth_user_file,
     AuthGroupFile => $auth_group_file,
+    AuthOrder => 'mod_auth_file.c',
 
     IfModules => {
       'mod_delay.c' => {
@@ -723,6 +727,7 @@ sub mlsd_ok_other_dir_bug4198 {
 
     AuthUserFile => $auth_user_file,
     AuthGroupFile => $auth_group_file,
+    AuthOrder => 'mod_auth_file.c',
 
     IfModules => {
       'mod_delay.c' => {
@@ -838,6 +843,7 @@ sub mlsd_ok_chrooted_dir {
 
     AuthUserFile => $setup->{auth_user_file},
     AuthGroupFile => $setup->{auth_group_file},
+    AuthOrder => 'mod_auth_file.c',
 
     DefaultRoot => '~',
 
@@ -966,6 +972,7 @@ sub mlsd_ok_empty_dir {
 
     AuthUserFile => $auth_user_file,
     AuthGroupFile => $auth_group_file,
+    AuthOrder => 'mod_auth_file.c',
 
     IfModules => {
       'mod_delay.c' => {
@@ -1106,6 +1113,7 @@ sub mlsd_ok_no_path {
 
     AuthUserFile => $auth_user_file,
     AuthGroupFile => $auth_group_file,
+    AuthOrder => 'mod_auth_file.c',
 
     IfModules => {
       'mod_delay.c' => {
@@ -1236,6 +1244,7 @@ sub mlsd_ok_glob {
 
     AuthUserFile => $auth_user_file,
     AuthGroupFile => $auth_group_file,
+    AuthOrder => 'mod_auth_file.c',
 
     IfModules => {
       'mod_delay.c' => {
@@ -1451,6 +1460,7 @@ sub mlsd_fails_enoent {
 
     AuthUserFile => $auth_user_file,
     AuthGroupFile => $auth_group_file,
+    AuthOrder => 'mod_auth_file.c',
 
     IfModules => {
       'mod_delay.c' => {
@@ -1588,6 +1598,7 @@ sub mlsd_fails_eperm {
 
     AuthUserFile => $auth_user_file,
     AuthGroupFile => $auth_group_file,
+    AuthOrder => 'mod_auth_file.c',
 
     IfModules => {
       'mod_delay.c' => {
@@ -1722,6 +1733,7 @@ sub mlsd_ok_hidden_file {
 
     AuthUserFile => $auth_user_file,
     AuthGroupFile => $auth_group_file,
+    AuthOrder => 'mod_auth_file.c',
 
     Directory => {
       '/' => {
@@ -1862,6 +1874,7 @@ sub mlsd_ok_path_with_spaces {
 
     AuthUserFile => $auth_user_file,
     AuthGroupFile => $auth_group_file,
+    AuthOrder => 'mod_auth_file.c',
 
     IfModules => {
       'mod_delay.c' => {
@@ -1992,6 +2005,7 @@ sub mlsd_nonascii_chars_bug3032 {
 
     AuthUserFile => $auth_user_file,
     AuthGroupFile => $auth_group_file,
+    AuthOrder => 'mod_auth_file.c',
 
     IfModules => {
       'mod_delay.c' => {
@@ -2131,6 +2145,7 @@ sub mlsd_symlink_showsymlinks_off_bug3318 {
 
     AuthUserFile => $setup->{auth_user_file},
     AuthGroupFile => $setup->{auth_group_file},
+    AuthOrder => 'mod_auth_file.c',
 
     ShowSymlinks => 'off',
 
@@ -2277,6 +2292,8 @@ sub mlsd_symlink_showsymlinks_on_bug3318 {
 
     AuthUserFile => $setup->{auth_user_file},
     AuthGroupFile => $setup->{auth_group_file},
+    AuthOrder => 'mod_auth_file.c',
+
     ShowSymlinks => 'on',
 
     IfModules => {
@@ -2431,6 +2448,7 @@ sub mlsd_symlink_showsymlinks_on_chrooted_bug4219 {
 
     AuthUserFile => $setup->{auth_user_file},
     AuthGroupFile => $setup->{auth_group_file},
+    AuthOrder => 'mod_auth_file.c',
 
     DefaultRoot => '~',
     ShowSymlinks => 'on',
@@ -2502,7 +2520,7 @@ sub mlsd_symlink_showsymlinks_on_chrooted_bug4219 {
 
       # Since ShowSymlinks is on, the type for test.lnk should indicate that
       # it's a symlink
-      $expected = 'OS.unix=symlink'; 
+      $expected = 'OS.unix=symlink';
       $got = $res->{'test.lnk'}->{type};
       $self->assert(qr/$expected/i, $got,
         test_msg("Expected '$expected', got '$got'"));
@@ -2587,6 +2605,7 @@ sub mlsd_symlink_showsymlinks_on_use_slink_chrooted_bug4219 {
 
     AuthUserFile => $setup->{auth_user_file},
     AuthGroupFile => $setup->{auth_group_file},
+    AuthOrder => 'mod_auth_file.c',
 
     DefaultRoot => '~',
     FactsOptions => 'UseSlink',
@@ -2755,6 +2774,7 @@ sub mlsd_symlinked_dir_bug3859 {
 
     AuthUserFile => $auth_user_file,
     AuthGroupFile => $auth_group_file,
+    AuthOrder => 'mod_auth_file.c',
 
     # Note: use the default ShowSymlinks setting
 
@@ -2923,6 +2943,7 @@ sub mlsd_symlinked_dir_showsymlinks_off_bug3859 {
 
     AuthUserFile => $auth_user_file,
     AuthGroupFile => $auth_group_file,
+    AuthOrder => 'mod_auth_file.c',
 
     ShowSymlinks => 'off',
 
@@ -3065,6 +3086,7 @@ sub mlsd_wide_dir {
 
     AuthUserFile => $setup->{auth_user_file},
     AuthGroupFile => $setup->{auth_group_file},
+    AuthOrder => 'mod_auth_file.c',
 
     IfModules => {
       'mod_delay.c' => {
@@ -3183,6 +3205,10 @@ sub mlsd_symlink_rel_path_chrooted_bug4322 {
     die("Can't symlink 'public_html' to './$dst_path': $!");
   }
 
+  unless (symlink("$dst_path", 'public_html2')) {
+    die("Can't symlink 'public_html2' to '$dst_path': $!");
+  }
+
   unless (chdir($cwd)) {
     die("Can't chdir to $cwd: $!");
   }
@@ -3204,6 +3230,8 @@ sub mlsd_symlink_rel_path_chrooted_bug4322 {
 
     AuthUserFile => $setup->{auth_user_file},
     AuthGroupFile => $setup->{auth_group_file},
+    AuthOrder => 'mod_auth_file.c',
+
     ShowSymlinks => 'on',
     DefaultRoot => '~',
 
@@ -3244,6 +3272,7 @@ sub mlsd_symlink_rel_path_chrooted_bug4322 {
       my $buf;
       $conn->read($buf, 8192, 30);
       eval { $conn->close() };
+      $client->quit();
 
       if ($ENV{TEST_VERBOSE}) {
         print STDERR "# MLSD:\n$buf\n";
@@ -3259,21 +3288,24 @@ sub mlsd_symlink_rel_path_chrooted_bug4322 {
       }
 
       my $count = scalar(keys(%$res));
-      my $expected = 10;
+      my $expected = 11;
       unless ($count == $expected) {
         die("MLSD returned wrong number of entries (expected $expected, got $count)");
       }
 
-      # public_html is a symlink to domains/test.oxilion.nl/public_html.
-      # According to RFC3659, the unique fact for both of these should thus
-      # be the same, since they are the same underlying object.
+      # public_html and public_html2 are symlinks to
+      # domains/test.oxilion.nl/public_html.  According to RFC3659, the unique
+      # fact for both of these should thus be the same, since they are the same
+      # underlying object.
 
       $expected = 'OS.unix=symlink';
       my $got = $res->{'public_html'}->{type};
       $self->assert(qr/$expected/i, $got,
         "Expected type fact '$expected', got '$got'");
 
-      $client->quit();
+      $got = $res->{'public_html2'}->{type};
+      $self->assert(qr/$expected/i, $got,
+        "Expected type fact '$expected', got '$got'");
     };
     if ($@) {
       $ex = $@;
@@ -3317,6 +3349,10 @@ sub mlsd_symlink_rel_path_subdir_chrooted_bug4322 {
     die("Can't symlink 'public_html' to './$dst_path': $!");
   }
 
+  unless (symlink("$dst_path", 'public_html2')) {
+    die("Can't symlink 'public_html2' to '$dst_path': $!");
+  }
+
   unless (chdir($cwd)) {
     die("Can't chdir to $cwd: $!");
   }
@@ -3338,6 +3374,8 @@ sub mlsd_symlink_rel_path_subdir_chrooted_bug4322 {
 
     AuthUserFile => $setup->{auth_user_file},
     AuthGroupFile => $setup->{auth_group_file},
+    AuthOrder => 'mod_auth_file.c',
+
     ShowSymlinks => 'on',
     DefaultRoot => '~',
 
@@ -3378,6 +3416,7 @@ sub mlsd_symlink_rel_path_subdir_chrooted_bug4322 {
       my $buf;
       $conn->read($buf, 8192, 30);
       eval { $conn->close() };
+      $client->quit();
 
       if ($ENV{TEST_VERBOSE}) {
         print STDERR "# MLSD:\n$buf\n";
@@ -3393,21 +3432,24 @@ sub mlsd_symlink_rel_path_subdir_chrooted_bug4322 {
       }
 
       my $count = scalar(keys(%$res));
-      my $expected = 4;
+      my $expected = 5;
       unless ($count == $expected) {
         die("MLSD returned wrong number of entries (expected $expected, got $count)");
       }
 
-      # public_html is a symlink to domains/test.oxilion.nl/public_html.
-      # According to RFC3659, the unique fact for both of these should thus
-      # be the same, since they are the same underlying object.
+      # public_html and public_html2 are symlinks to
+      # domains/test.oxilion.nl/public_html.  According to RFC3659, the unique
+      # fact for both of these should thus be the same, since they are the same
+      # underlying object.
 
       $expected = 'OS.unix=symlink';
       my $got = $res->{'public_html'}->{type};
       $self->assert(qr/$expected/i, $got,
         "Expected type fact '$expected', got '$got'");
 
-      $client->quit();
+      $got = $res->{'public_html2'}->{type};
+      $self->assert(qr/$expected/i, $got,
+        "Expected type fact '$expected', got '$got'");
     };
     if ($@) {
       $ex = $@;
@@ -3451,6 +3493,10 @@ sub mlsd_symlink_rel_path_subdir_cwd_chrooted_bug4322 {
     die("Can't symlink 'public_html' to './$dst_path': $!");
   }
 
+  unless (symlink("$dst_path", 'public_html2')) {
+    die("Can't symlink 'public_html2' to '$dst_path': $!");
+  }
+
   unless (chdir($cwd)) {
     die("Can't chdir to $cwd: $!");
   }
@@ -3472,6 +3518,8 @@ sub mlsd_symlink_rel_path_subdir_cwd_chrooted_bug4322 {
 
     AuthUserFile => $setup->{auth_user_file},
     AuthGroupFile => $setup->{auth_group_file},
+    AuthOrder => 'mod_auth_file.c',
+
     ShowSymlinks => 'on',
     DefaultRoot => '~',
 
@@ -3513,6 +3561,7 @@ sub mlsd_symlink_rel_path_subdir_cwd_chrooted_bug4322 {
       my $buf;
       $conn->read($buf, 8192, 30);
       eval { $conn->close() };
+      $client->quit();
 
       if ($ENV{TEST_VERBOSE}) {
         print STDERR "# MLSD:\n$buf\n";
@@ -3528,21 +3577,24 @@ sub mlsd_symlink_rel_path_subdir_cwd_chrooted_bug4322 {
       }
 
       my $count = scalar(keys(%$res));
-      my $expected = 4;
+      my $expected = 5;
       unless ($count == $expected) {
         die("MLSD returned wrong number of entries (expected $expected, got $count)");
       }
 
-      # public_html is a symlink to domains/test.oxilion.nl/public_html.
-      # According to RFC3659, the unique fact for both of these should thus
-      # be the same, since they are the same underlying object.
+      # public_html and public_html2 are symlinks to
+      # domains/test.oxilion.nl/public_html.  According to RFC3659, the unique
+      # fact for both of these should thus be the same, since they are the same
+      # underlying object.
 
       $expected = 'OS.unix=symlink';
       my $got = $res->{'public_html'}->{type};
       $self->assert(qr/$expected/i, $got,
         "Expected type fact '$expected', got '$got'");
 
-      $client->quit();
+      $got = $res->{'public_html2'}->{type};
+      $self->assert(qr/$expected/i, $got,
+        "Expected type fact '$expected', got '$got'");
     };
     if ($@) {
       $ex = $@;
